/* This routine will calculate the european price at a certain time slice  */
/* First generate the paths, and then compute the optionpayoff             */
/* Finally, generate the option price                                      */
/* Evaluate this option at T1 and T2, where T1>T2                          */
#include  <math.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "path.h"
#include "option.h"
double euro_payoff(double time,int paths,int underliers,
                   double **covariance, double *dividendRate,
                   double *growthRate, double K, double S) { 
  int path,underlier;
  double euro_sum;
  optionData od;
  int i,j;
#define PHRASE "The Maui financial modeling team"
  od.pData.meshPaths = paths;
  od.pData.underliers = underliers;
  od.pData.times = 2;
  od.pData.paths = paths;
  strcpy(od.pData.phrase,PHRASE);
  od.pData.clusterSize = 1;
  od.pData.clusterVariance = covariance[1][1]; /* we don't do clustering for this*/



  /* allocate the memory */
  pathDataAllocate(&(od.pData));
  optionDataAllocate(&od);


  for(i=0;i<od.pData.underliers;i++){
    for(j=0;j<od.pData.underliers;j++){
      od.pData.covariance[i][j] = covariance[i][j]*time;
      od.pData.growthRate[i] = growthRate[i]*time;
      od.pData.dividendRate[i] = dividendRate[i]*time;
    }
  }

  pathParamsInit(&(od.pData));
                                        /* Set all the initial prices */
  for(path=0; path<paths; path++){
    for(underlier=0; underlier < underliers; underlier++){
      od.pData.price[0][path][underlier] = S;
    }
  }
  pathDataFill(&(od.pData));
  optionDataFill(&od);
/* now we can just calculate the european prices */

  euro_sum = 0.0;
  for(path=0; path<paths; path++){
    euro_sum+=od.payoff[1][path];
  }
  euro_sum/=paths;
  
  pathDataDeallocate(&(od.pData));
  optionDataDeallocate(&od);
  return(euro_sum);
}
